''
'' VGA display 50xN (single cog) - user interface
''
''        Author: Marko Lukat
'' Last modified: 2011/12/13
''       Version: 0.1
''
CON
  columns  = 400 / 8
  rows     = 300 / font#height
  bcnt     = columns * rows

  rows_r   = (300 + font#height - 1) / font#height
  bcnt_r   = columns * rows_r

OBJ
  driver: "waitvid.50xN.driver.2048"
    font: "mikronauts8x10font"
    
VAR
  long  link[4]                                         ' driver mailbox
  byte  scrn[bcnt_r]                                    ' screen buffer

  long  x, y                                            ' cursor position
  long  flag
  
PUB null
'' This is not a top level object.

PUB init
  link{0} := @scrn[bcnt_r - columns]                    ' initial screen buffer and
  link[1] := font#height << 24 | font.addr              ' font definition (default palette)
  return driver.init(-1, @link{0})                      ' video driver and pixel generator

PUB str(addr)
'' Print a zero-terminated string
  repeat strsize(addr)
    out(byte[addr++])

PUB out(c)
'' Output a character
''
''     $00 = clear screen
''     $01 = home
''     $08 = backspace
''     $09 = tab (8 spaces per)
''     $0A = set X position (X follows)
''     $0B = set Y position (Y follows)
''     $0D = return
''  others = printable characters
  case flag~
    $00: case c
           $00: wordfill(@scrn{0}, $2020, constant(bcnt_r / 2))
                x := y := 0
           $01: x := y := 0
           $08: if x
                  x--
                elseif y
                  y--
                  x := columns -1
           $09: repeat
                  putc(" ")
                while x & 7
           $0A..$0B: flag := c
           $0D: newline
           other: putc(c)
    $0A: x := c // columns
    $0B: y := c // rows

PUB putc(c)
'' Output a character without filter
  scrn[constant(bcnt_r -1) - y * columns - x] := c
  if ++x == columns
    newline

PUB setPalette(addr)
'' Update palette
  link[2] := addr
  repeat while link[2]
  
PRI newline
  if ++y == rows
    y--
    wordmove(@scrn[columns], @scrn{0}, constant((rows_r -1) * columns / 2))
    if rows_r == rows
      wordfill(@scrn{0}, $2020, constant(columns / 2))
  x := 0

PUB getptr
  return @scrn 

PUB redefine_sg | i, j, ptr
  ptr := font.addr 
  repeat i from 0 to 4
    repeat j from 0 to 7
      bytemove(ptr + (i<<8) +  128 + (j<<4), @sg1, 16)
      bytemove(ptr + (i<<8) + 1408 + (j<<4), @sg2, 16)

DAT
sg1     byte  $00, $0F, $F0, $FF, $00, $0F, $F0, $FF, $00, $0F, $F0, $FF, $00, $0F, $F0, $FF
sg2     byte  $00, $00, $00, $00, $0F, $0F, $0F, $0F, $F0, $F0, $F0, $F0, $FF, $FF, $FF, $FF
